
## defining the data
mydata1 <- data.frame(read.csv("Hunan-contact-data.csv"))

nrow(mydata1)
summary(mydata1)

# "infected" - whether a contact was PCR positve (1=True, 0=False)   
# "agegroup" - age group the contact belongs to (expressed in years)     
# "typeC" - type of contact           
# "meas" - whether the exposure took place before or after the beginning of the implementatino of the strict social distancing measures (1=After, 0=Before)      
# "contact_gender2" - contact gender (1=Male, 0=Female) 
# "Travel_Wuhan" - whether the contact travelled/resided in Wuhan/Hubei (1=True, 0=False) 
# "cluster_id" - id of the cluster 

##### regression  analysis

## generalized linear model (logit)
## without accounting for clustering of the data 

logmod2 = glm(infected ~ agegroup + typeC  + contact_gender2 + Travel_Wuhan + meas,
             data = mydata1,
             family = binomial(link = "logit"))
summary(logmod2)
exp(logmod2$coefficients)
exp(confint(logmod2))

### estimate robust stundart errors

# install.packages("sandwich")
library(sandwich)

cov.m1 <- vcovHC(logmod2, type = "HC0")
std.err <- sqrt(diag(cov.m1))
q.val <- qnorm(0.975)
r.est <- cbind(
  Estimate = exp(coef(logmod2))
  , LL = exp(coef(logmod2) - q.val  * std.err)
  , UL = exp(coef(logmod2) + q.val  * std.err)
  , "Pr(>|z|) "= 2 * pnorm(abs(coef(logmod2)/std.err), lower.tail = FALSE)
)

r.est

### Conditional logit via Cox model
## accounts for cluster fixed effects

# install.packages("survival")
library("survival")

condlog1 = clogit(infected ~ agegroup + typeC  + contact_gender2 + Travel_Wuhan + meas +
                    strata(cluster_id), 
                  data = mydata1)
coefC = summary(condlog1)
confC = exp(confint(condlog1))
rescondlog1 = rbind(rep(NA, 4),
                    cbind(OR=coefC$coefficients[,"exp(coef)"], confC, 
                 pvalue=coefC$coefficients[,"Pr(>|z|)"]))


### Mixed effect logit model
## accounts for the random effects of clusters
## gives cluster-specific effects

# install.packages("lme4")
library("lme4")

str(mydata1)
feff.log = glmer(infected ~ agegroup + typeC  + contact_gender2 + Travel_Wuhan + meas +
                       (1|cluster_id), 
                     family = binomial(link = "logit"), data = mydata1,
                  nAGQ = 12,
                  control=glmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
summary(feff.log)


coef1 = exp(fixef(feff.log))
pp1 =  profile(feff.log)
conf1 = exp(confint(pp1))
res01 = broom::tidy(x = feff.log, conf.int = TRUE)


resglmm1 = cbind(OR=coef1, conf1[2:nrow(conf1),], 
                 pvalue=summary(feff.log)[[10]][,4])


### Generalize equation model
## accounts for correlation structure and clustering
## gives marginal effects

# install.packages("gee")
library("gee")

mod.gee1 =  gee(infected ~ agegroup + typeC  + contact_gender2 + Travel_Wuhan + meas,
                  id = cluster_id, corstr = "exchangeable", maxiter = 2e5,
                family = binomial(link = "logit"), data = mydata1)

summary(mod.gee1)
exp(mod.gee1$coefficients)
se = summary(mod.gee1)$coefficients[,"Robust S.E."]
resgee1 = cbind(OR=exp(mod.gee1$coefficients),
                exp(mod.gee1$coefficients - se * qnorm(0.975)),
                exp(mod.gee1$coefficients + se * qnorm(0.975)),
                pvalue=2 * pnorm(abs(coef(summary(mod.gee1))[,5]), lower.tail = FALSE))



## function to create the table of results

printingresults = function(t){
  AA = apply(t, 1, function(x)
    paste(format(x[1], digits=1, nsmall=2)," (",
          format(x[2], digits=1, nsmall=2),"-",
          format(x[3], digits=1, nsmall=2),")",
          ", p-value = ", format(x[4], digits=1, nsmall=4), sep=""))
  return(AA)
}

## create the table of results
topr0 = cbind(printingresults(rescondlog1),
              printingresults(resglmm1),
              printingresults(resgee1))
colnames(topr0) = c("Conditional Logit", "GLMM", "GEE")
write.csv(topr0,"results_print.csv")
      
