
##install.packages("deSolve")
library(deSolve)

## Compute the transmission rate (beta) using the next generation matrix approach
## the basic reproduction number, R0
## the inverse of the generation time, gamma
## the (adjusted) contact matrix, M - Mij represents the mean number of contacts that an individual of age i has with individuals of age j. Potentially adjusted by the relative susceptibility to infection by age
compute.beta <- function(R0,gamma,M){
    Eig = max(Re(eigen(M)$values))
    return(R0*gamma/Eig)
}

## Compute R0 using the next generation matrix approach given:
## the transmission rate, beta
## the inverse of the generation time, gamma
## the (adjusted) contact matrix, M - Mij represetns the mean number of contacts that an individual of age i has with individuals of age j. Potentially adjusted by the relative susceptibility to infection by age
compute.R0 <- function(beta,gamma,M){
    Eig = max(Re(eigen(M)$values))
    return(Eig*beta/gamma)
}

## definition of the age-structured SIR transmission model
SIR <- function(t,Y,z,parms){
    n = length(Y)/4
    if(n != nrow(z)){
        print("ERROR: dimensions do not match")
        stop()
    }

    beta = parms[1]
    gamma = parms[2]    

    S = Y[1:n]
    I = Y[(n+1):(2*n)]
    R = Y[(2*n+1):(3*n)]
    N = S+I+R

    Ys = NULL
    Yi = NULL
    Yr = NULL
    Yc = NULL
    for(i in 1:n){
        Ys[i] = -beta*sum(z[i,]*I/N)*S[i]
        Yi[i] = beta*sum(z[i,]*I/N)*S[i] - gamma*I[i]
        Yr[i] = gamma*I[i]
        Yc[i] = beta*sum(z[i,]*I/N)*S[i]
    }

    return(list(c(Ys,Yi,Yr,Yc)))
}

## simulation of the age-structured SIR transmission model
sim.sir <- function(beta,gamma,M,pop,final.time=365){
    yini = c(S = pop, I = c(1,rep(0,length(pop)-1)),
        R = rep(0,length(pop)), CC = rep(0,length(pop))) ## Initial conditions (fully susceptible population)
    res = ode(y = yini, func = SIR, times = seq(0, final.time, by = 1),
        z = M, parms=c(beta,gamma)) ## Simulation of the mdoel
    return( list(res=res) ) ## Return the dynamics of the model
}


#############
## Example ##
#############

## M = read contact matrix by age 
M = matrix(c(1,1,1,1),ncol=2) # example with a 2x2 contact matrix 

## pop = read the number of individuals in each age group
pop = c(1000,5000)

## Set parameters
gamma = 1/5.1 ## Inverse of the duration of the generation time
sus = c(0.4,1) # age-specific susceptibility to infection of individuals. (In this example we use 2 age groups)

## Adjust the contact matrix to account for the age-specific susceptibility to infection
M1 = apply(M, 2, function(x) x*sus)

## Compute the transmission rate (beta), assuming R0=2.5
beta = compute.beta(R0=2.5,gamma,M1)

###############
## Simulation of the age-structured SIR model for final.time days
###############
dynamics = sim.sir(beta=beta, gamma=gamma, M=M, pop=pop, final.time=365)
print(dynamics$res)
print(res$res[365,]) ## time, number of susceptible individuals for each age group (S), number of infectious individuals for each age group (I), number of removed individuals for each age group (R), cumulative number of cases for each age group (CC) 

##############
## Calculation of R0 from the next generation matrix approach
##############
beta = 0.5
R0 = compute.R0(beta=beta, gamma=gamma, M1) 
print(R0)




